# Generated by Django 4.1.7 on 2023-08-16 05:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('nets_core', '0006_verificationcode_device'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verificationcode',
            name='device',
            field=models.ForeignKey(blank=True, max_length=150, null=True, on_delete=django.db.models.deletion.CASCADE, to='nets_core.userdevice'),
        ),
    ]
