NETS_CORE_PROTECTED_FIELDS = [
    'password',
    'is_active',
    'enabled',
    'staff',
    'superuser',
    'verified',
    'deleted',
    'token',
    'auth',
    'perms',
    'groups',
    'ip',
    'email',
    'doc',
    'permissions',
    'date_joined',
    'last_login',
    'verified',
    'updated_fields'
]